//=================== Klassen-Definition: PyPLOT ========================================
// PyPLOT-Klasse fuer elementare Python-Graphik im C++Wrapper matplotlibcpp
// Matrizen werden spaltenweise gespeichert !!!
// --------------------------------------------------------------------------------------
// -> Hinweise zur Installation in Visual-Studio:
// https://stackoverflow.com/questions/66507618/
//                 running-python-matplotlibcpp-in-visual-studio-2019
// -> Hinweise zur Dokumentation:
// Python-Graphik: https://matplotlib-cpp.readthedocs.io/en/latest/
// -> Hinweise zur Benutzung:
// matplotlibcpp: https://github.com/lava/matplotlib-cpp
// ======================================================================================
// Copyright: Univ.-Prof. Dr.-Ing. Udo F. Meissner, Erkrath/Darmstadt
// Lehr- und Studienprogramm - kommerzielle Nutzung nicht gestattet. 
// Der Autor uebernimmt keine Haftung fuer Unvollkommenheit und Fehler
// Version:  2023
// Compiler: C++ 
// ================ Include-Files =======================================================
#include "pch.h"
// -------------
#include "PyPLOTclass.h"
//
// #include "PyPlot.h"
// #include "MATRIX.h"
// #include "VEKTOR.h"
// ================================================
// Python-Graphik mittels C++ Wrapper matplotlibcpp 
// ================================================
namespace plt = matplotlibcpp;
//
// ============================================================================
// public -> Oeffentliche Mitgliedsfunktionen
// ==========================================
//
// Konstruktor -> PyPLOT py(fakt0, fakt1, kreis)
// =============================================
PyPLOT::PyPLOT(double fakt0, double fakt1, double kreis) : faktx(fakt0), fakty(fakt1), kreis_r(kreis/1000.0)
{
//*** std::cout << "PyPLOT::Konstruktor: neue Instanz von PyPLOT erzeugen = " << endl;
//            -----------------------------------------------------
//*** cout << "Konstruktor PyPLOT: " << "fakt0 = " << fakt0 << "fakt1 = " << fakt1 << " kreis = " << kreis << endl;
};//-------------------------------------------------------------------------------------------------------------
//
// Destruktor -> delete &per;
// ==========================
PyPLOT::~PyPLOT(void)
{
//*** std::cout<<"PyPLOT::Destruktor: Instanz von PyPLOT loeschen " << endl;
};//-------------------------------------------------------
//
// Bildtitel ausgeben
// ==================
void PyPLOT::bild_titel(string text)
{
    namespace plt = matplotlibcpp;
//
    plt::suptitle(text);
}; // -------------------------------------------------------------------------
//
// Bild zeigen
// ===========
void PyPLOT::zeige_bild()
{
	namespace plt = matplotlibcpp;
	//
	plt::show();
}; // -------------------------------------------------------------------------
//
// Punkt (Typ 2) zeichnen -> py.PyPunkt
// // MATRIX A_punkt ist dreidimensional und enthlt mehrere Punkte in der dritten Dimension
// =========================================================================================
void PyPLOT::punkt(MATRIX& A_punkt, int punkt_a, int auge, string farbe, string marker, string bezeichner, double zahl)
{
	// ================================================
	// Python-Graphik mittels C++ Wrapper matplotlibcpp 
	// ================================================
	namespace plt = matplotlibcpp;
	//
		// Bildkoordinaten 
	std::vector<double> vx(1), vy(1);
	//
	vx.at(0) = A_punkt(0, auge, punkt_a) * faktx;
	vy.at(0) = A_punkt(1, auge, punkt_a) * fakty;
	//
	//*** char c; 
	//*** std::cout << vx.at(0) << vy.at(0) << endl;
	//*** std::cout << "Pause vor Plot Punkt:" << endl; std::cin >> c;
	//
	// plot marker
	plt::plot(vx, vy, { {"color", farbe}, { "marker", marker }, { "linestyle", " "} });
	//
	// plot text
	char* text_zahl;

	strstream puffer;
	// Puffer leeren
	puffer.setf(ios::unitbuf);
	//
	puffer << bezeichner;
//  --------------------- 
	//
	// plot Zahl neben A_punkt schreiben
	if (zahl != 0.0)
	{
		// Zahlengenauigkeit
		puffer.width(5);
		puffer.precision(1);
		puffer.setf(ios::fixed);         //*** puffer.setf(ios::scientific);
		//
		// abstand_auge ausgeben
		puffer << "= " << zahl << " cm";
//      --------------------------------
	};
	// String-Ende NULL = '\0'
	puffer << '\0';
//  ---------------
	// 
	text_zahl = puffer.str();
	// 
	// Puffer leeren
	puffer.setf(ios::unitbuf);
	//
	// Kreis um A zeichnen entfllt wegen Marker
	plt::text(vx.at(0) + kreis_r, vy.at(0) + kreis_r, text_zahl); // text -> Farbe nicht mglich !?
	//                                                               ==============================                                                      
//*** plt::text(vx.at(0) + kreis_r, vy.at(0) + kreis_r, text_zahl, color=farbe); // text -> Farbe nicht mglich !?
//***                                                                               ==============================
};	// --------------------------------------------------------------
//
// Punkt (Typ 1) zeichnen -> py.PyPunkt
// // MATRIX A_punkt ist zweidimensional und enthlt nur einen Punkt
// =================================================================
void PyPLOT::punkt(MATRIX& A_punkt, int auge, string farbe, string marker, string bezeichner, double zahl)
{
	// ================================================
	// Python-Graphik mittels C++ Wrapper matplotlibcpp 
	// ================================================
        //
	punkt(A_punkt, 0, auge,  farbe, marker, bezeichner, zahl);
        //
}; // -------------------------------------------------------------------------
//
// Linie (Typ 2) zeichnen -> py.PyLinie
// MATRIX A_punkt, E_punkt sind dreidimensional und enthalten mehrere Punkte in der dritten Dimension 
// ==================================================================================================
void PyPLOT::linie(MATRIX& A_punkt, int punkt_a, MATRIX& E_punkt, int punkt_e, int auge, string farbe, string lstil, string bezeichner)
{

	//*** matplotcppcpp mit C++11_Erweiterung !!! ***********************************************
	//*** #include <cmat> 
	//*** #include <matplot/matplot.h>
	//*** using namespace matplot;
	// ******************************************************************************************

	// ================================================
	// Python-Graphik mittels C++ Wrapper matplotlibcpp 
	// ================================================
	namespace plt = matplotlibcpp;
	//
	// Bildkoordinaten 
	std::vector<double> vx(2), vy(2);
	//
	vx.at(0) = A_punkt(0, auge, punkt_a) * faktx;   vx.at(1) = E_punkt(0, auge, punkt_e) * faktx;
	vy.at(0) = A_punkt(1, auge, punkt_a) * fakty;   vy.at(1) = E_punkt(1, auge, punkt_e) * fakty;
	//                                   
	//*** char c; 
	//*** std::cout << x.at(0) << y.at(0) << endl;
	//*** std::cout << "Pause vor Plot Punkt:" << endl; std::cin >> c;
	//
	// plot Linie
	plt::plot(vx, vy, { {"color", farbe}, {"marker", " "}, {"linestyle", lstil} });
	// 
	// plot Text am Ende
	plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, bezeichner); // text -> Farbe nicht mglich !?
    //                                                                ==============================
//*** plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, bezeichner, 'color'=farbe); // text -> Farbe nicht mglich !?
//***                                                                                  ==============================
//*****************************************************************************************************
//*** matplotcppcpp
//*** auto t = text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, bezeichner)->color (farbe).font_size (14);
//***************************************************************************************************** 
//*** Python
//*** plt.text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, bezeichner,     {'color' : 'farbe'});
//*****************************************************************************************************
}; // -------------------------------------------------------------------------------------------------      
//
// Linie (Typ 1) zeichnen -> py.PyLinie
// // MATRIX A_punkt, E_punkt sind zweidimensional und enthalten nur einen Punkt
// =============================================================================
void PyPLOT::linie(MATRIX& A_punkt, MATRIX& E_punkt, int auge, string farbe, string lstil, string bezeichner)
{
	// ================================================
	// Python-Graphik mittels C++ Wrapper matplotlibcpp 
	// ================================================
	//
	linie(A_punkt, 0, E_punkt, 0, auge, farbe, lstil, bezeichner);
    //
}; // -------------------------------------------------------------------------
//
// Dreibein zeichnen -> anzahl_der Beine=py.PyDreibein
// ===================================================
int PyPLOT::dreibein(MATRIX& A_punkt, MATRIX& E_punkte, int auge, string farbe, string lstil, string bezeichner)
{
	// Python-Graphik mittels C++ Wrapper matplotlibcpp 
	// ================================================
	namespace plt = matplotlibcpp;
	//
		// Bildkoordinaten 
	std::vector<double> vx(2), vy(2);
	//
	vx.at(0) = A_punkt(0, auge) * faktx;       
	vy.at(0) = A_punkt(1, auge) * fakty;       
	//
	//*** char c; 
	//*** std::cout << vx.at(0) << vy.at(0) << endl;
	//*** std::cout << "Pause vor Plot Punkt:" << endl; std::cin >> c;
	//
	//
	// Anzahl der Vektoren feststellen
	int dimi, augen, punkte, diml;
	E_punkte.dimension_ermitteln(dimi, augen, punkte, diml);
	//*** std::cout << "punkte = " << punkte << endl;
	// ----------------------------------------------------------------------------
	// Text-Bezeichnungen
	string text0; string text1; string text2;
	if (bezeichner == "E") { text0 = "$e{_0}$"; text1 = "$e{_1}$"; text2 = "$e{_2}$"; };
	if (bezeichner == "G") { text0 = "$g{_0}$"; text1 = "$g{_1}$"; text2 = "$g{_2}$"; };
	if (bezeichner == "g") { text0 = "$g{^0}$"; text1 = "$g{^1}$"; text2 = "$g{^2}$"; };
	// ----------------------------------------------------------------------------
	// Vektor 0
	vx.at(1) = E_punkte(0, auge, 0) * faktx;
	vy.at(1) = E_punkte(1, auge, 0) * fakty;
	// plot Linie
	plt::plot(vx, vy, { {"color", farbe}, {"marker", " "}, {"linestyle", lstil} });
	// 
	// plot Text am Ende
	plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text0); // text -> Farbe nicht mglich !?
	//                                                           ==============================
//*** plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text0, color=farbe); // text -> Farbe nicht mglich !?   
//***                                                                           ==============================
	if (punkte == 1) return punkte;
	// ----------------------------------------------------------------------------      
	// Vektor 1
	vx.at(1) = E_punkte(0, auge, 1) * faktx;
	vy.at(1) = E_punkte(1, auge, 1) * fakty;
	// plot Linie
	plt::plot(vx, vy, { {"color", farbe}, {"marker", " "}, {"linestyle", lstil} });
	// 
	// plot Text am Ende
	plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text1); // text -> Farbe nicht mglich !?
	//                                                           ==============================
//*** plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text1, color=farbe); // text -> Farbe nicht mglich !?
//***                                                                           ==============================          
	if (punkte == 2) return punkte;
	// ----------------------------------------------------------------------------
	// Vektor 2
	vx.at(1) = E_punkte(0, auge, 2) * faktx;
	vy.at(1) = E_punkte(1, auge, 2) * fakty;
	// plot Linie
	plt::plot(vx, vy, { {"color", farbe}, {"marker", " "}, {"linestyle", lstil} });
	// 
	// plot Text am Ende
	plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text2); // text -> Farbe nicht mglich !?
	//                                                           ==============================
//*** plt::text(vx.at(1) + kreis_r, vy.at(1) + kreis_r, text2, color=farbe); // text -> Farbe nicht mglich !?
//***                                                                           ==============================                                                    
	if (punkte == 3) return punkte;
	//
	return punkte;
}; // -------------------------------------------------------------------------
